# frozen_string_literal: true

#
# Stolen from https://github.com/rails/rails/blob/94ca3e0a571dba0fe41ca18d61634c5f3aa11209/activesupport/lib/active_support/core_ext/class/attribute.rb
# and https://github.com/rails/rails/blob/94ca3e0a571dba0fe41ca18d61634c5f3aa11209/activesupport/lib/active_support/core_ext/module/remove_method.rb

# rubocop:disable Metrics/AbcSize, Metrics/CyclomaticComplexity, Metrics/MethodLength
# rubocop:disable Metrics/PerceivedComplexity
# rubocop:disable Style/Send, Style/GuardClause, Metrics/BlockNesting
# rubocop:disable Style/OptionHash
module  AppleCore
module  Extensions
module  Class
  module ClassMethods
    def class_attribute(method_name, options = {})
      instance_reader = options.fetch(:instance_accessor, true) &&
                        options.fetch(:instance_reader, true)
      instance_writer = options.fetch(:instance_accessor, true) &&
                        options.fetch(:instance_writer, true)

      remove_possible_singleton_method(method_name)
      define_singleton_method(method_name) { options[:default] }

      ivar = "@#{method_name}"

      remove_possible_singleton_method("#{method_name}=")
      define_singleton_method("#{method_name}=") do |val|
        singleton_class.class_eval do
          remove_possible_method(method_name)
          define_method(method_name) { val }
        end

        if singleton_class?
          class_eval do
            remove_possible_method(method_name)
            define_method(method_name) do
              if instance_variable_defined? ivar
                instance_variable_get ivar
              else
                singleton_class.send method_name
              end
            end
          end
        end
        val
      end

      if instance_reader
        remove_possible_method method_name
        define_method(method_name) do
          if instance_variable_defined?(ivar)
            instance_variable_get ivar
          else
            self.class.public_send method_name
          end
        end
      end

      if instance_writer
        remove_possible_method "#{method_name}="
        attr_writer method_name
      end
    end
  end

  def self.included(base)
    base.extend(ClassMethods)
  end
end
end
end
# rubocop:enable Style/OptionHash
# rubocop:enable Style/Send, Style/GuardClause, Metrics/BlockNesting
# rubocop:enable Metrics/PerceivedComplexity
# rubocop:enable Metrics/AbcSize, Metrics/CyclomaticComplexity, Metrics/MethodLength
