# frozen_string_literal: true

require 'apple_core/transforms/enumerable'

module AppleCore
module Refinements
module Array
refine ::Array do
  def delete!(default = nil, &block)
    index_of_item = index(&block)

    return default unless index_of_item.present?

    delete_at index_of_item
  end

  def compact_blank!
    delete_if(&:blank?)
  end

  def deep_strip!
    each_with_index do |value, index|
      if value.respond_to?(:strip)
        self[index] = value.strip
      elsif value.respond_to?(:deep_strip!)
        self[index] = value.deep_strip!
      end
    end
  end

  def extract_options
    if last.is_a?(::Hash) && last.instance_of?(::Hash)
      last
    else
      {}
    end
  end

  def deep_transform_keys(&block)
    Transforms::Enumerable.deep_transform_keys(self, &block)
  end

  def deep_transform_values(&block)
    Transforms::Enumerable.deep_transform_values(nil, self, &block)
  end
end
end
end
end
