# frozen_string_literal: true

module AppleCore
module Refinements
module DeepDup
refine ::Array do
  unless method_defined?(:deep_dup)
    def deep_dup
      map { |i| i.respond_to?(:deep_dup) ? i.deep_dup : i.dup }
    end
  end
end

refine ::Object do
  unless method_defined?(:deep_dup)
    def deep_dup
      dup
    end
  end
end

refine ::Hash do
  unless method_defined?(:deep_dup)
    def deep_dup
      dup.tap do |hash|
        each_pair do |key, value|
          if key.frozen? && key.is_a?(::String)
            hash[key] = value.deep_dup
          else
            hash.delete(key)
            hash[key.deep_dup] = value.deep_dup
          end
        end
      end
    end
  end
end
end
end
end

module ActiveSupport
class  HashWithIndifferentAccess < Hash
  using ::AppleCore::Refinements::DeepDup

  unless method_defined?(:deep_dup)
    def deep_dup
      dup.tap do |hash|
        each_pair do |key, value|
          if key.frozen? && key.is_a?(::String)
            hash[key] = value.deep_dup
          else
            hash.delete(key)
            hash[key.deep_dup] = value.deep_dup
          end
        end
      end
    end
  end
end
end
