# frozen_string_literal: true

require 'apple_core/transforms/enumerable'

module AppleCore
module Refinements
module Hash
refine ::Hash do
  def deep_strip!
    each do |key, value|
      if value.respond_to?(:strip)
        self[key] = value.strip
      elsif value.respond_to?(:deep_strip!)
        self[key] = value.deep_strip!
      end
    end
  end

  def deep_underscore_keys
    Transforms::Enumerable.deep_underscore_keys(self)
  end

  def deep_transform_keys(&block)
    Transforms::Enumerable.deep_transform_keys(self, &block)
  end

  def deep_transform_values(&block)
    Transforms::Enumerable.deep_transform_values(nil, self, &block)
  end

  unless method_defined?(:slice)
    def slice(*keys)
      keys.each_with_object({}) do |k, hash|
        hash[k] = self[k] if has_key?(k)
      end
    end
  end

  unless method_defined?(:slice!)
    def slice!(*keys)
      omit              = slice(*self.keys - keys)

      hash              = slice(*keys)
      hash.default      = default
      hash.default_proc = default_proc if default_proc

      replace(hash)

      omit
    end
  end

  def symbolize_keys
    result = {}

    each_key do |key|
      result[key.to_sym] = self[key]
    end

    result
  end
end
end
end
end
