# frozen_string_literal: true

module AppleCore
module Refinements
module QueryString
refine ::Array do
  unless method_defined?(:to_param)
    def to_param
      map(&:to_param).join('/')
    end

    def to_query(key)
      prefix = "#{key}[]"

      return nil.to_query(prefix) if empty?

      map { |value| value.to_query(prefix) }
        .join('&')
    end
  end
end

refine ::FalseClass do
  def to_param
    self
  end
end

refine ::Hash do
  unless method_defined?(:to_param)
    def to_param(namespace = nil)
      map { |key, value|
        unless (value.is_a?(Hash) || value.is_a?(Array)) && value.empty?
          value.to_query(namespace ? "#{namespace}[#{key}]" : key)
        end
      }
        .compact
        .sort!
        .join('&')
    end

    alias_method :to_query, :to_param
  end
end

refine ::NilClass do
  unless method_defined?(:to_param)
    def to_param
      self
    end
  end
end

refine ::Object do
  unless method_defined?(:to_param)
    def to_param
      to_s
    end

    def to_query(key)
      "#{CGI.escape(key.to_param)}=#{CGI.escape(to_param.to_s)}"
    end
  end
end

refine ::TrueClass do
  def to_param
    self
  end
end
end
end
end

module ActiveSupport
class  HashWithIndifferentAccess < Hash
  using ::AppleCore::Refinements::QueryString

  unless method_defined?(:to_param)
    def to_param
      dup.tap do |hash|
        each_pair do |key, value|
          if key.frozen? && key.is_a?(::String)
            hash[key] = value.to_param
          else
            hash.delete(key)
            hash[key.to_param] = value.to_param
          end
        end
      end
    end
  end
end
end
